import base64
import hashlib
import hmac
from io import BytesIO
from pathlib import Path
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    NoReturn,
    Optional,
    Union,
    cast,
)
from typing_extensions import override
from urllib.parse import urlencode

from nonebot.adapters import Bot as BaseBot
from nonebot.drivers import Request, Response
from nonebot.message import handle_event
from nonebot.utils import escape_tag

from pydantic import parse_obj_as
import rsa

from .config import BotInfo
from .event import AddQuickEmoticonEvent, Event, SendMessageEvent
from .exception import (
    ActionFailed,
    BotNotAdded,
    InsufficientPermission,
    InvalidBotAuthInfo,
    InvalidMemberBotAccessToken,
    InvalidRequest,
    NetworkError,
    PermissionDenied,
    UnknownServerError,
    UnsupportedMsgType,
)
from .message import (
    BadgeSegment,
    ComponentsSegment,
    ImageSegment,
    LinkSegment,
    MentionAllSegement,
    MentionRobotSegement,
    MentionUserSegement,
    Message,
    MessageSegment,
    PanelSegment,
    PostSegment,
    PreviewLinkSegment,
    QuoteSegment,
    RoomLinkSegment,
    TextSegment,
)
from .models import (
    ApiResponse,
    CheckMemberBotAccessTokenReturn,
    Color,
    Command,
    Component,
    ContentType,
    Emoticon,
    Group,
    GroupRoom,
    ImageMessageContent,
    ImageUploadResult,
    Link,
    Member,
    MemberListReturn,
    MemberRoleDetail,
    MentionedAll,
    MentionedInfo,
    MentionType,
    MessageContentInfo,
    Panel,
    Permission,
    PostMessageContent,
    Robot,
    Room,
    RoomSort,
    TextEntity,
    TextMessageContent,
    TextStyle,
    UploadImageParamsReturn,
    Villa,
    VillaRoomLink,
    WebsocketInfo,
)
from .utils import API, get_img_extenion, get_img_md5, log

if TYPE_CHECKING:
    from .adapter import Adapter


def _check_reply(bot: "Bot", event: SendMessageEvent):
    """检查事件是否有引用消息，如果有则删除。

    参数:
        bot: Bot对象
        event: 事件
    """
    if event.content.quote is not None:
        event.message = event.message.exclude("quote")
    if not event.message:
        event.message.append(MessageSegment.text(""))


def _check_at_me(bot: "Bot", event: SendMessageEvent):
    """检查事件是否和机器人有关，如果有关则设置 to_me 为 True，并删除消息中的 at 信息。

    参数:
        bot: Bot对象
        event: 事件

    """
    # 目前只能收到艾特机器人的消息，所以永远为 True
    # if (
    #     event.content.mentioned_info
    #     and bot.self_id in event.content.mentioned_info.user_id_list
    # ):
    #     event.to_me = True

    def _is_at_me_seg(segment: MessageSegment) -> bool:
        return (
            segment.type == "mention_robot"
            and segment.data["mention_robot"].bot_id == bot.self_id
        )

    message = event.get_message()
    if not message:
        message.append(MessageSegment.text(""))

    deleted = False
    if _is_at_me_seg(message[0]):
        message.pop(0)
        deleted = True
        if message and message[0].type == "text":
            message[0].data["text"] = message[0].data["text"].lstrip("\xa0").lstrip()
            if not message[0].data["text"]:
                del message[0]

    if not deleted:
        # check the last segment
        i = -1
        last_msg_seg = message[i]
        if (
            last_msg_seg.type == "text"
            and not last_msg_seg.data["text"].strip()
            and len(message) >= 2
        ):
            i -= 1
            last_msg_seg = message[i]

        if _is_at_me_seg(last_msg_seg):
            deleted = True
            del message[i:]

    if not message:
        message.append(MessageSegment.text(""))


class Bot(BaseBot):
    """
    大别野协议 Bot 适配。
    """

    adapter: "Adapter"

    @override
    def __init__(
        self,
        adapter: "Adapter",
        self_id: str,
        bot_info: BotInfo,
        **kwargs: Any,
    ) -> None:
        super().__init__(adapter, self_id)
        self.adapter: Adapter = adapter
        self.bot_secret: str = bot_info.bot_secret
        self.bot_secret_encrypt = hmac.new(
            bot_info.pub_key.encode(),
            bot_info.bot_secret.encode(),
            hashlib.sha256,
        ).hexdigest()
        self.pub_key = rsa.PublicKey.load_pkcs1_openssl_pem(bot_info.pub_key.encode())
        self.verify_event = bot_info.verify_event
        self._bot_info: Optional[Robot] = None
        self._ws_info: Optional[WebsocketInfo] = None
        self._ws_squence: int = 0

    @override
    def __repr__(self) -> str:
        return f"Bot(type={self.type!r}, self_id={self.self_id!r})"

    @override
    def __getattr__(self, name: str) -> NoReturn:
        raise AttributeError(
            f'"{self.__class__.__name__}" object has no attribute "{name}"',
        )

    @property
    def nickname(self) -> str:
        """Bot 昵称"""
        if not self._bot_info:
            raise ValueError(f"Bot {self.self_id} hasn't received any events yet.")
        return self._bot_info.template.name

    @property
    def commands(self) -> Optional[List[Command]]:
        """Bot 命令预设命令列表"""
        if not self._bot_info:
            raise ValueError(f"Bot {self.self_id} hasn't received any events yet.")
        return self._bot_info.template.commands

    @property
    def description(self) -> Optional[str]:
        """Bot 介绍描述"""
        if not self._bot_info:
            raise ValueError(f"Bot {self.self_id} hasn't received any events yet.")
        return self._bot_info.template.desc

    @property
    def avatar_icon(self) -> str:
        """Bot 头像图标地址"""
        if not self._bot_info:
            raise ValueError(f"Bot {self.self_id} hasn't received any events yet.")
        return self._bot_info.template.icon

    @property
    def current_villd_id(self) -> int:
        if not self._bot_info:
            raise ValueError(f"Bot {self.self_id} hasn't received any events yet.")
        return self._bot_info.villa_id

    @property
    def ws_info(self) -> WebsocketInfo:
        if self._ws_info is None:
            raise RuntimeError(f"Bot {self.self_id} is not connected!")
        return self._ws_info

    @ws_info.setter
    def ws_info(self, ws_info: WebsocketInfo):
        self._ws_info = ws_info

    async def handle_event(self, event: Event):
        """处理事件"""
        if isinstance(event, SendMessageEvent):
            _check_at_me(self, event)
            _check_reply(self, event)
        await handle_event(self, event)

    def _verify_signature(
        self,
        body: str,
        bot_sign: str,
    ) -> bool:
        sign = base64.b64decode(bot_sign)
        sign_data = urlencode(
            {"body": body.rstrip("\n"), "secret": self.bot_secret},
        ).encode()
        try:
            rsa.verify(sign_data, sign, self.pub_key)
        except rsa.VerificationError:
            return False
        return True

    def get_authorization_header(
        self,
        villa_id: Optional[int] = None,
    ) -> Dict[str, str]:
        """Bot 鉴权凭证请求头

        参数:
            villa_id: 大别野ID

        返回:
            Dict[str, str]: 请求头
        """
        return {
            "x-rpc-bot_id": self.self_id,
            "x-rpc-bot_secret": self.bot_secret_encrypt,
            "x-rpc-bot_villa_id": str(villa_id or ""),
        }

    async def _handle_respnose(self, response: Response) -> Any:
        if not response.content:
            raise NetworkError("API request error when parsing response")
        resp = ApiResponse.parse_raw(response.content)
        if resp.retcode == 0:
            return resp.data
        if resp.retcode == -502:
            raise UnknownServerError(resp)
        if resp.retcode == -1:
            raise InvalidRequest(resp)
        if resp.retcode == 10318001:
            raise InsufficientPermission(resp)
        if resp.retcode == 10322002:
            raise BotNotAdded(resp)
        if resp.retcode == 10322003:
            raise PermissionDenied(resp)
        if resp.retcode == 10322004:
            raise InvalidMemberBotAccessToken(resp)
        if resp.retcode == 10322005:
            raise InvalidBotAuthInfo(resp)
        if resp.retcode == 10322006:
            raise UnsupportedMsgType(resp)
        raise ActionFailed(response.status_code, resp)

    async def _request(self, request: Request):
        try:
            resp = await self.adapter.request(request)
            log(
                "TRACE",
                f"API status_code:{resp.status_code} content: {escape_tag(str(resp.content))}",  # noqa: E501
            )
        except Exception as e:
            raise NetworkError("API request failed") from e
        return await self._handle_respnose(resp)

    async def send_to(
        self,
        villa_id: int,
        room_id: int,
        message: Union[str, Message, MessageSegment],
    ) -> str:
        """向指定房间发送消息

        参数:
            villa_id: 大别野 ID
            room_id: 房间 ID
            message: 消息

        返回:
            str: 消息 ID
        """
        message = message if isinstance(message, Message) else Message(message)
        content_info = await self.parse_message_content(message)
        if isinstance(content_info.content, PostMessageContent):
            object_name = "MHY:Post"
        elif isinstance(content_info.content, ImageMessageContent):
            object_name = "MHY:Image"
        else:
            object_name = "MHY:Text"
        return await self.send_message(
            villa_id=villa_id,
            room_id=room_id,
            object_name=object_name,
            msg_content=content_info.json(
                by_alias=True,
                exclude_none=True,
                ensure_ascii=False,
            ),
        )

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs: Any,
    ) -> str:
        """发送消息

        参数:
            event: 事件
            message: 消息
            mention_sender: 是否@消息发送者. 默认为 False.
            quote_message: 是否引用原消息. 默认为 False.

        异常:
            RuntimeError: 事件不是消息事件时抛出

        返回:
            str: 消息ID
        """
        if not isinstance(event, (SendMessageEvent, AddQuickEmoticonEvent)):
            raise RuntimeError("Event cannot be replied to!")
        message = message if isinstance(message, Message) else Message(message)
        if kwargs.pop("mention_sender", False) or kwargs.pop("at_sender", False):
            message.insert(
                0,
                MessageSegment.mention_user(
                    user_id=int(event.get_user_id()),
                    user_name=(
                        event.content.user.name
                        if isinstance(event, SendMessageEvent)
                        else None
                    ),
                    villa_id=event.villa_id,
                ),
            )
        if kwargs.pop("quote_message", False) or kwargs.pop("reply_message", False):
            message += MessageSegment.quote(event.msg_uid, event.send_at)
        return await self.send_to(
            villa_id=event.villa_id,
            room_id=event.room_id,
            message=message,
        )

    async def parse_message_content(self, message: Message) -> MessageContentInfo:
        """将适配器的Message对象转为大别野发送所需要的MessageContentInfo对象

        参数:
            message: 消息

        返回:
            MessageContentInfo: 消息内容对象
        """
        quote = image = post = badge = preview_link = panel = None
        if quote_seg := cast(Optional[List[QuoteSegment]], message["quote"] or None):
            quote = quote_seg[-1].data["quote"]
        if image_seg := cast(
            Optional[List[ImageSegment]],
            message["image"] or None,
        ):
            image = image_seg[-1].data["image"]
        if post_seg := cast(Optional[List[PostSegment]], message["post"] or None):
            post = post_seg[-1].data["post"]
        if badge_seg := cast(Optional[List[BadgeSegment]], message["badge"] or None):
            badge = badge_seg[-1].data["badge"]
        if preview_link_seg := cast(
            Optional[List[PreviewLinkSegment]],
            message["preview_link"] or None,
        ):
            preview_link = preview_link_seg[-1].data["preview_link"]
        if panel_seg := cast(Optional[List[PanelSegment]], message["panel"] or None):
            panel = panel_seg[-1].data["panel"]
        if panel is None:
            components: List[Component] = []
            for com in message["components"]:
                components.extend(cast(ComponentsSegment, com).data["components"])
            if components:
                panel = _parse_components(components)

        def cal_len(x):
            return len(x.encode("utf-16")) // 2 - 1

        message = message.exclude("quote", "image", "post", "badge", "preview_link")
        message_text = ""
        message_offset = 0
        entities: List[TextEntity] = []
        mentioned = MentionedInfo(type=MentionType.PART)
        for seg in message:
            try:
                if isinstance(seg, TextSegment):
                    seg_text = seg.data["text"]
                    length = cal_len(seg_text)
                    for style in {"bold", "italic", "underline", "strikethrough"}:
                        if seg.data[style]:
                            entities.append(
                                TextEntity(
                                    offset=message_offset,
                                    length=length,
                                    entity=TextStyle(font_style=style),  # type: ignore
                                ),
                            )
                elif isinstance(seg, MentionAllSegement):
                    mention_all: MentionedAll = seg.data["mention_all"]
                    seg_text = f"@{mention_all.show_text} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=mention_all,
                        ),
                    )
                    mentioned.type = MentionType.ALL
                elif isinstance(seg, MentionRobotSegement):
                    mention_robot = seg.data["mention_robot"]
                    seg_text = f"@{mention_robot.bot_name} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=mention_robot,
                        ),
                    )
                    mentioned.user_id_list.append(mention_robot.bot_id)
                elif isinstance(seg, MentionUserSegement):
                    mention_user = seg.data["mention_user"]
                    if mention_user.user_name is None:
                        if not seg.data["villa_id"]:
                            raise ValueError("cannot get user name without villa_id")
                        # 需要调用API获取被@的用户的昵称
                        user = await self.get_member(
                            villa_id=seg.data["villa_id"],
                            uid=int(mention_user.user_id),
                        )
                        seg_text = f"@{user.basic.nickname} "
                        mention_user.user_name = user.basic.nickname
                    else:
                        seg_text = f"@{mention_user.user_name} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=mention_user,
                        ),
                    )
                    mentioned.user_id_list.append(str(mention_user.user_id))
                elif isinstance(seg, RoomLinkSegment):
                    room_link: VillaRoomLink = seg.data["room_link"]
                    if room_link.room_name is None:
                        # 需要调用API获取房间的名称
                        room = await self.get_room(
                            villa_id=int(room_link.villa_id),
                            room_id=int(room_link.room_id),
                        )
                        seg_text = f"#{room.room_name} "
                        room_link.room_name = room.room_name
                    else:
                        seg_text = f"#{room_link.room_name} "
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(
                            offset=message_offset,
                            length=length,
                            entity=room_link,
                        ),
                    )
                elif isinstance(seg, LinkSegment):
                    link: Link = seg.data["link"]
                    seg_text = link.show_text
                    length = cal_len(seg_text)
                    entities.append(
                        TextEntity(offset=message_offset, length=length, entity=link),
                    )
                else:
                    continue
                message_offset += length
                message_text += seg_text
            except Exception as e:
                log("WARNING", "error when parse message content", e)

        if not (mentioned.type == MentionType.ALL and mentioned.user_id_list):
            mentioned = None

        if not (message_text or entities):
            if preview_link or badge:
                content = TextMessageContent(
                    text="\u200b",
                    preview_link=preview_link,
                    badge=badge,
                    images=[image] if image else None,
                )
            elif image:
                content = image
            elif post:
                content = post
            else:
                raise ValueError("message content is empty")
        else:
            content = TextMessageContent(
                text=message_text,
                entities=entities,
                images=[image] if image else None,
                preview_link=preview_link,
                badge=badge,
            )

        return MessageContentInfo(
            content=content,
            mentionedInfo=mentioned,
            quote=quote,
            panel=panel,
        )

    @API
    async def check_member_bot_access_token(
        self,
        *,
        villa_id: int,
        token: str,
    ) -> CheckMemberBotAccessTokenReturn:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "checkMemberBotAccessToken",
            headers=self.get_authorization_header(
                villa_id,
            ),
            json={"token": token},
        )
        return parse_obj_as(
            CheckMemberBotAccessTokenReturn,
            await self._request(request),
        )

    @API
    async def get_villa(self, villa_id: int) -> Villa:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getVilla",
            headers=self.get_authorization_header(villa_id),
        )
        return parse_obj_as(Villa, (await self._request(request))["villa"])

    @API
    async def get_member(
        self,
        *,
        villa_id: int,
        uid: int,
    ) -> Member:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getMember",
            headers=self.get_authorization_header(villa_id),
            json={"uid": uid},
        )
        return parse_obj_as(Member, (await self._request(request))["member"])

    @API
    async def get_villa_members(
        self,
        *,
        villa_id: int,
        offset: int,
        size: int,
    ) -> MemberListReturn:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getVillaMembers",
            headers=self.get_authorization_header(villa_id),
            json={"offset": offset, "size": size},
        )
        return parse_obj_as(MemberListReturn, await self._request(request))

    @API
    async def delete_villa_member(
        self,
        *,
        villa_id: int,
        uid: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "deleteVillaMember",
            headers=self.get_authorization_header(villa_id),
            json={"uid": uid},
        )
        await self._request(request)

    @API
    async def pin_message(
        self,
        *,
        villa_id: int,
        msg_uid: str,
        is_cancel: bool,
        room_id: int,
        send_at: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "pinMessage",
            headers=self.get_authorization_header(villa_id),
            json={
                "msg_uid": msg_uid,
                "is_cancel": is_cancel,
                "room_id": room_id,
                "send_at": send_at,
            },
        )
        await self._request(request)

    @API
    async def recall_message(
        self,
        *,
        villa_id: int,
        msg_uid: str,
        room_id: int,
        msg_time: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "recallMessage",
            headers=self.get_authorization_header(villa_id),
            json={"msg_uid": msg_uid, "room_id": room_id, "msg_time": msg_time},
        )
        await self._request(request)

    @API
    async def send_message(
        self,
        *,
        villa_id: int,
        room_id: int,
        object_name: str,
        msg_content: Union[str, MessageContentInfo],
    ) -> str:
        if isinstance(msg_content, MessageContentInfo):
            content = msg_content.json(
                by_alias=True,
                exclude_none=True,
                ensure_ascii=False,
            )
        else:
            content = msg_content
        request = Request(
            method="POST",
            url=self.adapter.base_url / "sendMessage",
            headers=self.get_authorization_header(villa_id),
            json={
                "room_id": room_id,
                "object_name": object_name,
                "msg_content": content,
            },
        )
        return (await self._request(request))["bot_msg_id"]

    @API
    async def create_component_template(
        self,
        *,
        panel: Panel,
    ) -> int:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "createComponentTemplate",
            headers=self.get_authorization_header(),
            json={
                "panel": panel.json(exclude_none=True, ensure_ascii=False),
            },
        )
        return (await self._request(request))["template_id"]

    @API
    async def create_group(
        self,
        *,
        villa_id: int,
        group_name: str,
    ) -> int:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "createGroup",
            headers=self.get_authorization_header(villa_id),
            json={"group_name": group_name},
        )
        return (await self._request(request))["group_id"]

    @API
    async def edit_group(
        self,
        *,
        villa_id: int,
        group_id: int,
        group_name: str,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "editGroup",
            headers=self.get_authorization_header(villa_id),
            json={"group_id": group_id, "group_name": group_name},
        )
        await self._request(request)

    @API
    async def delete_group(
        self,
        *,
        villa_id: int,
        group_id: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "deleteGroup",
            headers=self.get_authorization_header(villa_id),
            json={"group_id": group_id},
        )
        await self._request(request)

    @API
    async def get_group_list(self, *, villa_id: int) -> List[Group]:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getGroupList",
            headers=self.get_authorization_header(villa_id),
        )
        return parse_obj_as(List[Group], (await self._request(request))["list"])

    @API
    async def sort_group_list(
        self,
        *,
        villa_id: int,
        group_ids: List[int],
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "sortGroupList",
            headers=self.get_authorization_header(villa_id),
            json={"villa_id": villa_id, "group_ids": group_ids},
        )
        await self._request(request)

    @API
    async def edit_room(
        self,
        *,
        villa_id: int,
        room_id: int,
        room_name: str,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "editRoom",
            headers=self.get_authorization_header(villa_id),
            json={"room_id": room_id, "room_name": room_name},
        )
        await self._request(request)

    @API
    async def delete_room(
        self,
        *,
        villa_id: int,
        room_id: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "deleteRoom",
            headers=self.get_authorization_header(villa_id),
            json={"room_id": room_id},
        )
        await self._request(request)

    @API
    async def get_room(
        self,
        *,
        villa_id: int,
        room_id: int,
    ) -> Room:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getRoom",
            headers=self.get_authorization_header(villa_id),
            json={"room_id": room_id},
        )
        return parse_obj_as(Room, (await self._request(request))["room"])

    @API
    async def get_villa_group_room_list(
        self,
        *,
        villa_id: int,
    ) -> List[GroupRoom]:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getVillaGroupRoomList",
            headers=self.get_authorization_header(villa_id),
        )
        return parse_obj_as(
            List[GroupRoom],
            (await self._request(request))["list"],
        )

    @API
    async def sort_room_list(
        self,
        *,
        villa_id: int,
        room_list: List[RoomSort],
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "sortRoomList",
            headers=self.get_authorization_header(villa_id),
            json={
                "villa_id": villa_id,
                "room_list": [room.dict() for room in room_list],
            },
        )
        await self._request(request)

    @API
    async def operate_member_to_role(
        self,
        *,
        villa_id: int,
        role_id: int,
        uid: int,
        is_add: bool,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "operateMemberToRole",
            headers=self.get_authorization_header(villa_id),
            json={"role_id": role_id, "uid": uid, "is_add": is_add},
        )
        await self._request(request)

    @API
    async def create_member_role(
        self,
        *,
        villa_id: int,
        name: str,
        color: Color,
        permissions: List[Permission],
    ) -> int:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "createMemberRole",
            headers=self.get_authorization_header(villa_id),
            json={"name": name, "color": str(color), "permissions": permissions},
        )
        return (await self._request(request))["id"]

    @API
    async def edit_member_role(
        self,
        *,
        villa_id: int,
        role_id: int,
        name: str,
        color: Color,
        permissions: List[Permission],
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "editMemberRole",
            headers=self.get_authorization_header(villa_id),
            json={
                "id": role_id,
                "name": name,
                "color": str(color),
                "permissions": permissions,
            },
        )
        await self._request(request)

    @API
    async def delete_member_role(
        self,
        *,
        villa_id: int,
        role_id: int,
    ) -> None:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "deleteMemberRole",
            headers=self.get_authorization_header(villa_id),
            json={"id": role_id},
        )
        await self._request(request)

    @API
    async def get_member_role_info(
        self,
        *,
        villa_id: int,
        role_id: int,
    ) -> MemberRoleDetail:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getMemberRoleInfo",
            headers=self.get_authorization_header(villa_id),
            json={"role_id": role_id},
        )
        return parse_obj_as(
            MemberRoleDetail,
            (await self._request(request))["role"],
        )

    @API
    async def get_villa_member_roles(
        self,
        *,
        villa_id: int,
    ) -> List[MemberRoleDetail]:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getVillaMemberRoles",
            headers=self.get_authorization_header(villa_id),
        )
        return parse_obj_as(
            List[MemberRoleDetail],
            (await self._request(request))["list"],
        )

    @API
    async def get_all_emoticons(self) -> List[Emoticon]:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getAllEmoticons",
        )
        return parse_obj_as(List[Emoticon], (await self._request(request))["list"])

    @API
    async def audit(
        self,
        *,
        villa_id: int,
        audit_content: str,
        pass_through: str,
        room_id: int,
        uid: int,
        content_type: ContentType,
    ) -> str:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "audit",
            headers=self.get_authorization_header(villa_id),
            json={
                "audit_content": audit_content,
                "pass_through": pass_through,
                "room_id": room_id,
                "uid": uid,
                "content_type": content_type,
            },
        )
        return (await self._request(request))["audit_id"]

    @API
    async def transfer_image(
        self,
        *,
        url: str,
    ) -> str:
        request = Request(
            method="POST",
            url=self.adapter.base_url / "transferImage",
            headers=self.get_authorization_header(),
            json={
                "url": url,
            },
        )
        return (await self._request(request))["new_url"]

    @API
    async def get_upload_image_params(
        self,
        *,
        md5: str,
        ext: str,
    ) -> UploadImageParamsReturn:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getUploadImageParams",
            headers=self.get_authorization_header(),
            params={
                "md5": md5,
                "ext": ext,
            },
        )
        return parse_obj_as(UploadImageParamsReturn, await self._request(request))

    async def upload_image(
        self,
        image: Union[bytes, BytesIO, Path],
        ext: Optional[str] = None,
    ) -> ImageUploadResult:
        if isinstance(image, Path):
            image = image.read_bytes()
        elif isinstance(image, BytesIO):
            image = image.getvalue()
        img_md5 = get_img_md5(image)
        ext = ext or get_img_extenion(image)
        if ext is None:
            raise ValueError("cannot guess image extension")
        upload_params = await self.get_upload_image_params(
            md5=img_md5,
            ext=ext,
        )
        request = Request(
            "POST",
            url=upload_params.params.host,
            data=upload_params.params.to_upload_data(),
            files={"file": image},
        )
        return parse_obj_as(ImageUploadResult, await self._request(request))

    async def get_websocket_info(
        self,
    ) -> WebsocketInfo:
        request = Request(
            method="GET",
            url=self.adapter.base_url / "getWebsocketInfo",
            headers=self.get_authorization_header(),
        )
        return parse_obj_as(WebsocketInfo, await self._request(request))


def _parse_components(components: List[Component]) -> Optional[Panel]:
    small_total = [[]]
    mid_total = [[]]
    big_total = []
    for com in components:
        com_lenght = len(com.text.encode("utf-8"))
        if com_lenght <= 0:
            log("warning", f"component {com.id} text is empty, ignore")
        elif com_lenght <= 6:
            small_total[-1].append(com)
            if len(small_total[-1]) >= 3:
                small_total.append([])
        elif com_lenght <= 12:
            mid_total[-1].append(com)
            if len(mid_total[-1]) >= 2:
                mid_total.append([])
        elif com_lenght <= 30:
            big_total[-1].append([com])
        else:
            log("warning", f"component {com.id} text is too long, ignore")
    if not small_total[-1]:
        small_total.pop()
    small_total = small_total or None
    if not mid_total[-1]:
        mid_total.pop()
    mid_total = mid_total or None
    big_total = big_total or None
    if small_total or mid_total or big_total:
        return Panel(
            small_component_group_list=small_total,
            mid_component_group_list=mid_total,
            big_component_group_list=big_total,
        )
    return None



from typing import TYPE_CHECKING, Optional

from nonebot.exception import (
    ActionFailed as BaseActionFailed,
    AdapterException,
    ApiNotAvailable as BaseApiNotAvailable,
    NetworkError as BaseNetworkError,
    NoLogException as BaseNoLogException,
)

if TYPE_CHECKING:
    from betterproto import Message

    from .api import ApiResponse


class VillaAdapterException(AdapterException):
    def __init__(self):
        super().__init__("Villa")


class NoLogException(BaseNoLogException, VillaAdapterException):
    pass


class ReconnectError(VillaAdapterException):
    def __init__(self, payload: Optional["Message"] = None):
        super().__init__()
        self.payload = payload

    def __repr__(self) -> str:
        if self.payload is None:
            return "Receive unexpected data, reconnect."
        return f"Reconnect because of {self.payload}"

    def __str__(self) -> str:
        return self.__repr__()


class DisconnectError(VillaAdapterException):
    ...


class ActionFailed(BaseActionFailed, VillaAdapterException):
    def __init__(self, status_code: int, response: "ApiResponse"):
        super().__init__()
        self.status_code = status_code
        self.response = response

    def __repr__(self) -> str:
        return (
            f"<{self.__class__.__name__}: {self.status_code}, "
            f"retcode={self.response.retcode}, "
            f"message={self.response.message}, "
            f"data={self.response.data}>"
        )

    def __str__(self):
        return self.__repr__()


class UnknownServerError(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(-502, response)


class InvalidRequest(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(-1, response)


class InsufficientPermission(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10318001, response)


class BotNotAdded(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10322002, response)


class PermissionDenied(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10322003, response)


class InvalidMemberBotAccessToken(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10322004, response)


class InvalidBotAuthInfo(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10322005, response)


class UnsupportedMsgType(ActionFailed):
    def __init__(self, response: "ApiResponse"):
        super().__init__(10322006, response)


class NetworkError(BaseNetworkError, VillaAdapterException):
    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg: Optional[str] = msg
        """错误原因"""

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, VillaAdapterException):
    def __init__(self, api: str):
        super().__init__()
        self.api = api

    def __repr__(self):
        return f"<ApiNotAvailable api={self.api}>"

    def __str__(self):
        return self.__repr__()



from warnings import warn

warn(
    "nonebot.adapters.villa.api is DEPRECATED! "
    "Please use nonebot.adapters.villa.models instead.",
    DeprecationWarning,
    stacklevel=2,
)
from .models import *



import asyncio
import json
import time
from typing import Any, Dict, List, Literal, Optional, cast
from typing_extensions import override

from nonebot.adapters import Adapter as BaseAdapter
from nonebot.drivers import (
    URL,
    Driver,
    HTTPClientMixin,
    HTTPServerSetup,
    Request,
    Response,
    ReverseMixin,
    WebSocket,
    WebSocketClientMixin,
)
from nonebot.exception import WebSocketClosed
from nonebot.utils import escape_tag

from pydantic import parse_obj_as

from .bot import Bot
from .config import BotInfo, Config
from .event import (
    Event,
    event_classes,
    pre_handle_event_websocket,
    pre_handle_webhook_event,
)
from .exception import ApiNotAvailable, DisconnectError, ReconnectError
from .models import WebsocketInfo
from .payload import (
    BizType,
    HeartBeat,
    HeartBeatReply,
    KickOff,
    Login,
    LoginReply,
    Logout,
    LogoutReply,
    Payload,
    RobotEvent,
    Shutdown,
)
from .utils import API, log


class Adapter(BaseAdapter):
    bots: Dict[str, Bot]

    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.villa_config: Config = Config(**self.config.dict())
        self.tasks: List[asyncio.Task] = []
        self.ws: Dict[str, WebSocket] = {}
        self.base_url: URL = URL("https://bbs-api.miyoushe.com/vila/api/bot/platform")
        self._setup()

    @classmethod
    @override
    def get_name(cls) -> Literal["Villa"]:
        return "Villa"

    def _setup(self):
        self.driver.on_startup(self._forward_http)
        self.driver.on_startup(self._start_forward)
        self.driver.on_shutdown(self._stop_forwards)

    async def _forward_http(self):
        webhook_bots = [
            bot_info
            for bot_info in self.villa_config.villa_bots
            if bot_info.connection_type == "webhook"
        ]
        if webhook_bots and not (
            isinstance(self.driver, ReverseMixin)
            and isinstance(self.driver, HTTPClientMixin)
        ):
            raise RuntimeError(
                (
                    f"Current driver {self.config.driver}"
                    "doesn't support connections!"
                    "Villa Adapter Webhook need a "
                    "ReverseMixin and HTTPClientMixin to work."
                ),
            )
        for bot_info in webhook_bots:
            if not bot_info.callback_url:
                log(
                    "WARNING",
                    f"<r>Missing callback url for bot {bot_info.bot_id}</r>, "
                    "bot will not be connected",
                )
                continue
            bot = Bot(self, bot_info.bot_id, bot_info)
            self.bot_connect(bot)
            log("INFO", f"<y>Bot {bot.self_id} connected</y>")
            http_setup = HTTPServerSetup(
                URL(bot_info.callback_url),
                "POST",
                f"大别野 {bot_info.bot_id} HTTP",
                self._handle_http,
            )
            self.setup_http_server(http_setup)

    async def _handle_http(self, request: Request) -> Response:
        if data := request.content:
            json_data = json.loads(data)
            if payload_data := json_data.get("event"):
                try:
                    event = parse_obj_as(
                        event_classes,
                        pre_handle_webhook_event(payload_data),
                    )
                    bot_id = event.bot_id
                    if (bot := self.bots.get(bot_id, None)) is None:
                        if (
                            bot_info := next(
                                (
                                    bot
                                    for bot in self.villa_config.villa_bots
                                    if bot.bot_id == bot_id
                                ),
                                None,
                            )
                        ) is not None:
                            bot = Bot(
                                self,
                                bot_info.bot_id,
                                bot_info,
                            )
                            self.bot_connect(bot)
                            log("INFO", f"<y>Bot {bot.self_id} connected</y>")
                        else:
                            log(
                                "WARNING",
                                (
                                    f"<r>Missing bot secret for bot {bot_id}</r>, "
                                    "event will not be handle"
                                ),
                            )
                            return Response(
                                200,
                                content=json.dumps(
                                    {"retcode": 0, "message": "NoneBot2 Get it!"},
                                ),
                            )
                    bot = cast(Bot, bot)
                    if bot.verify_event and (
                        (bot_sign := request.headers.get("x-rpc-bot_sign")) is None
                        or not bot._verify_signature(
                            data.decode() if isinstance(data, bytes) else str(data),
                            bot_sign,
                        )
                    ):
                        log("WARNING", f"Received invalid signature {bot_sign}.")
                        return Response(401, content="Invalid Signature")
                    bot._bot_info = event.robot
                except Exception as e:
                    log(
                        "WARNING",
                        f"Failed to parse event {escape_tag(str(payload_data))}",
                        e,
                    )
                else:
                    asyncio.create_task(bot.handle_event(event))
                return Response(
                    200,
                    content=json.dumps({"retcode": 0, "message": "NoneBot2 Get it!"}),
                )
            return Response(415, content="Invalid Request Body")
        return Response(415, content="Invalid Request Body")

    async def _start_forward(self) -> None:
        ws_bots = [
            bot_info
            for bot_info in self.villa_config.villa_bots
            if bot_info.connection_type == "websocket"
        ]
        if ws_bots and not (
            isinstance(self.driver, HTTPClientMixin)
            and isinstance(self.driver, WebSocketClientMixin)
        ):
            raise RuntimeError(
                f"Current driver {self.config.driver}"
                "doesn't support connections!"
                "Villa Adapter Websocket need a "
                "HTTPClientMixin and WebSocketClientMixin to work.",
            )
        for bot_config in ws_bots:
            if bot_config.connection_type == "websocket":
                bot = Bot(self, bot_config.bot_id, bot_config)
                ws_info = await bot.get_websocket_info()
                self.tasks.append(
                    asyncio.create_task(
                        self._forward_ws(bot, bot_config, ws_info),
                    ),
                )

    async def _forward_ws(
        self,
        bot: Bot,
        bot_config: BotInfo,
        ws_info: WebsocketInfo,
    ) -> None:
        request = Request(method="GET", url=URL(ws_info.websocket_url), timeout=30.0)
        heartbeat_task: Optional["asyncio.Task"] = None
        while True:
            try:
                async with self.websocket(request) as ws:
                    log(
                        "DEBUG",
                        "WebSocket Connection to"
                        f" {escape_tag(ws_info.websocket_url)} established",
                    )
                    try:
                        # 登录
                        result = await self._login(bot, ws, bot_config, ws_info)
                        if not result:
                            await asyncio.sleep(3.0)
                            continue

                        # 开启心跳
                        heartbeat_task = asyncio.create_task(
                            self._heartbeat(bot, ws),
                        )

                        # 处理事件
                        await self._loop(bot, ws)
                    except DisconnectError as e:
                        raise e
                    except ReconnectError as e:
                        log("ERROR", str(e), e)
                    except WebSocketClosed as e:
                        log(
                            "ERROR",
                            "<r><bg #f8bbd0>WebSocket Closed</bg #f8bbd0></r>",
                            e,
                        )
                    except Exception as e:
                        log(
                            "ERROR",
                            (
                                "<r><bg #f8bbd0>Error while process data from"
                                f" websocket {escape_tag(ws_info.websocket_url)}. "
                                "Trying to reconnect...</bg #f8bbd0></r>"
                            ),
                            e,
                        )
                    finally:
                        if bot.self_id in self.bots:
                            bot._ws_squence = 0
                            self.ws.pop(bot.self_id)
                            self.bot_disconnect(bot)
                        if heartbeat_task:
                            heartbeat_task.cancel()
                            heartbeat_task = None
            except DisconnectError:
                return
            except Exception as e:
                log(
                    "ERROR",
                    (
                        "<r><bg #f8bbd0>Error while setup websocket to"
                        f" {escape_tag(ws_info.websocket_url)}. "
                        "Trying to reconnect...</bg #f8bbd0></r>"
                    ),
                    e,
                )
                await asyncio.sleep(3.0)

    async def _stop_forwards(self) -> None:
        await asyncio.gather(
            *[
                self._stop_forward(self.bots[bot[0]], bot[1], task)
                for bot, task in zip(self.ws.items(), self.tasks)
            ],
        )

    async def _stop_forward(
        self,
        bot: Bot,
        ws: WebSocket,
        task: "asyncio.Task",
    ) -> None:
        await self._logout(bot, ws)
        await asyncio.sleep(1.0)
        if not task.done():
            task.cancel()

    async def _login(
        self,
        bot: Bot,
        ws: WebSocket,
        bot_config: BotInfo,
        ws_info: WebsocketInfo,
    ):
        try:
            login = Login(
                ws_info.uid,
                str(bot_config.test_villa_id or 0)
                + f".{bot_config.bot_secret}.{bot_config.bot_id}",
                ws_info.platform,
                ws_info.app_id,
                ws_info.device_id,
            )
            log("TRACE", f"Sending Login {escape_tag(repr(login))}")
            await ws.send_bytes(login.to_bytes_package(bot._ws_squence))
            bot._ws_squence += 1

        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Error while sending Login</bg #f8bbd0></r>",
                e,
            )
            return None
        login_reply = await self.receive_payload(ws)
        if not isinstance(login_reply, LoginReply):
            log(
                "ERROR",
                "Received unexpected event while login: "
                f"{escape_tag(repr(login_reply))}",
            )
            return None
        if login_reply.code == 0:
            bot.ws_info = ws_info
            if bot.self_id not in self.bots:
                self.bot_connect(bot)
                self.ws[bot.self_id] = ws
                log(
                    "INFO",
                    f"<y>Bot {escape_tag(bot.self_id)}</y> connected",
                )
            return True
        return None

    async def _logout(self, bot: Bot, ws: WebSocket):
        try:
            await ws.send_bytes(
                Logout(
                    uid=bot.ws_info.uid,
                    platform=bot.ws_info.platform,
                    app_id=bot.ws_info.app_id,
                    device_id=bot.ws_info.device_id,
                ).to_bytes_package(bot._ws_squence),
            )
            bot._ws_squence += 1
        except Exception as e:
            log("WARNING", "Error while sending logout, Ignored!", e)

    async def _heartbeat(self, bot: Bot, ws: WebSocket):
        while True:
            await asyncio.sleep(20.0)
            timestamp = str(int(time.time() * 1000))
            log("TRACE", f"Heartbeat {timestamp}")
            try:
                await ws.send_bytes(
                    HeartBeat(timestamp).to_bytes_package(bot._ws_squence),
                )
                bot._ws_squence += 1
            except Exception as e:
                log("WARNING", "Error while sending heartbeat, Ignored!", e)

    async def _loop(self, bot: Bot, ws: WebSocket):
        while True:
            payload = await self.receive_payload(ws)
            if not payload:
                raise ReconnectError
            if isinstance(payload, HeartBeatReply):
                log("TRACE", f"Heartbeat ACK in {payload.server_timestamp}")
                continue
            if isinstance(payload, (LogoutReply, KickOff)):
                if isinstance(payload, KickOff):
                    log("WARNING", f"Bot {bot.self_id} kicked off by server: {payload}")
                    raise DisconnectError
                log("INFO", f"<y>Bot {bot.self_id} disconnected: {payload}</y>")
                if bot.self_id in self.bots:
                    self.ws.pop(bot.self_id)
                    self.bot_disconnect(bot)
            if isinstance(payload, Event):
                bot._bot_info = payload.robot
                asyncio.create_task(bot.handle_event(payload))

    @staticmethod
    async def receive_payload(ws: WebSocket):
        payload = Payload.from_bytes(await ws.receive_bytes())
        if payload.biz_type in {BizType.P_LOGIN, BizType.P_LOGOUT, BizType.P_HEARTBEAT}:
            if payload.biz_type == BizType.P_LOGIN:
                payload = LoginReply.FromString(payload.body_data)
            elif payload.biz_type == BizType.P_LOGOUT:
                payload = LogoutReply.FromString(payload.body_data)
            else:
                payload = HeartBeatReply.FromString(payload.body_data)
            if payload.code != 0:
                if isinstance(payload, LogoutReply):
                    log("WARNING", f"Error when logout from server: {payload}")
                    return payload
                raise ReconnectError(payload)
        elif payload.biz_type == BizType.P_KICK_OFF:
            payload = KickOff.FromString(payload.body_data)
        elif payload.biz_type == BizType.SHUTDOWN:
            payload = Shutdown()
        elif payload.biz_type == BizType.EVENT:
            event_data = RobotEvent.FromString(payload.body_data)
            return parse_obj_as(
                event_classes,
                pre_handle_event_websocket(event_data),
            )
        else:
            raise ReconnectError
        log("TRACE", f"Received payload: {escape_tag(repr(payload))}")
        return payload

    @override
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        log("DEBUG", f"Calling API <y>{api}</y>")
        log("TRACE", f"With Data <y>{escape_tag(str(data))}</y>")
        api_handler: Optional[API] = getattr(bot.__class__, api, None)
        if api_handler is None:
            raise ApiNotAvailable(api)
        return await api_handler(bot, **data)



from dataclasses import dataclass
from enum import IntEnum
import struct
from typing import Dict, List, Literal

import betterproto
from pydantic import BaseModel


class BizType(IntEnum):
    UNKNOWN = 0
    EXCHANGE_KEY = 1
    HEARTBEAT = 2
    LOGIN = 3
    LOGOUT = 4
    P_EXCHANGE_KEY = 5
    P_HEARTBEAT = 6
    P_LOGIN = 7
    P_LOGOUT = 8
    KICK_OFF = 51
    SHUTDOWN = 52
    P_KICK_OFF = 53
    ROOM_ENTER = 60
    ROOM_LEAVE = 61
    ROOM_CLOSE = 62
    ROOM_MSG = 63
    EVENT = 30001


class Payload(BaseModel):
    # magic: Literal[0xBABEFACE] = 0xBABEFACE
    # data_len: int
    # header_len: int
    id: int
    flag: Literal[1, 2]
    biz_type: BizType
    app_id: Literal[104] = 104
    body_data: bytes

    @classmethod
    def from_bytes(cls, data: bytes):
        magic, data_len = struct.unpack("<II", data[:8])
        header_len, packet_id, flag, biz_type, app_id = struct.unpack(
            "<IQIIi",
            data[8:32],
        )
        body_data = data[32 : 8 + data_len]
        return cls(
            id=packet_id,
            flag=flag,
            biz_type=BizType(biz_type),
            app_id=app_id,
            body_data=body_data,
        )

    def to_bytes(self) -> bytes:
        changeable = (
            struct.pack(
                "<IQIIi",
                24,
                self.id,
                self.flag,
                self.biz_type,
                self.app_id,
            )
            + self.body_data
        )

        return (
            struct.pack(
                "<II",
                0xBABEFACE,
                len(changeable),
            )
            + changeable
        )


@dataclass
class HeartBeat(betterproto.Message):
    """心跳请求命令字"""

    # 客户端时间戳，精确到ms
    client_timestamp: str = betterproto.string_field(1)

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_HEARTBEAT,
            body_data=self.SerializeToString(),
        ).to_bytes()


@dataclass
class HeartBeatReply(betterproto.Message):
    """心跳返回"""

    # 错误码 非0表示失败
    code: int = betterproto.int32_field(1)
    # 服务端时间戳，精确到ms
    server_timestamp: int = betterproto.uint64_field(2)


@dataclass
class Login(betterproto.Message):
    """登录命令"""

    # 长连接侧唯一id，uint64格式
    uid: int = betterproto.uint64_field(1)
    # 用于业务后端验证的token
    token: str = betterproto.string_field(2)
    # 客户端操作平台枚举
    platform: int = betterproto.int32_field(3)
    # 业务所在客户端应用标识，用于在同一个客户端隔离不同业务的长连接通道。
    app_id: int = betterproto.int32_field(4)
    device_id: str = betterproto.string_field(5)
    # 区域划分字段，通过uid+app_id+platform+region四个字段唯一确定一条长连接
    region: str = betterproto.string_field(6)
    # 长连内部的扩展字段，是个map
    meta: Dict[str, str] = betterproto.map_field(
        7,
        betterproto.TYPE_STRING,
        betterproto.TYPE_STRING,
    )

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_LOGIN,
            body_data=self.SerializeToString(),
        ).to_bytes()


@dataclass
class LoginReply(betterproto.Message):
    """登录命令返回"""

    # 错误码 非0表示失败
    code: int = betterproto.int32_field(1)
    # 错误信息
    msg: str = betterproto.string_field(2)
    # 服务端时间戳，精确到ms
    server_timestamp: int = betterproto.uint64_field(3)
    # 唯一连接ID
    conn_id: int = betterproto.uint64_field(4)


@dataclass
class Logout(betterproto.Message):
    """登出命令字"""

    # 长连接侧唯一id，uint64格式
    uid: int = betterproto.uint64_field(1)
    # 客户端操作平台枚举
    platform: int = betterproto.int32_field(2)
    # 业务所在客户端应用标识，用于在同一个客户端隔离不同业务的长连接通道。
    app_id: int = betterproto.int32_field(3)
    # 客户端设备唯一标识
    device_id: str = betterproto.string_field(4)
    # 区域划分字段，通过uid+app_id+platform+region四个字段唯一确定一条长连接
    region: str = betterproto.string_field(5)

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_LOGOUT,
            body_data=self.SerializeToString(),
        ).to_bytes()


@dataclass
class LogoutReply(betterproto.Message):
    """登出命令返回"""

    # 错误码 非0表示失败
    code: int = betterproto.int32_field(1)
    # 错误信息
    msg: str = betterproto.string_field(2)
    # 连接id
    conn_id: int = betterproto.uint64_field(3)


@dataclass
class CommonReply(betterproto.Message):
    """通用返回"""

    # 错误码 非0表示失败
    code: int = betterproto.int32_field(1)
    # 错误信息
    msg: str = betterproto.string_field(2)


@dataclass
class KickOff(betterproto.Message):
    """踢出连接协议"""

    # 踢出原因状态码
    code: int = betterproto.int32_field(1)
    # 状态码对应的文案
    reason: str = betterproto.string_field(2)


@dataclass
class Shutdown(betterproto.Message):
    """服务关机"""


@dataclass
class RobotTemplate(betterproto.Message):
    id: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)
    desc: str = betterproto.string_field(3)
    icon: str = betterproto.string_field(4)
    commands: List["RobotTemplateCommand"] = betterproto.message_field(5)
    custom_settings: List["RobotTemplateCustomSetting"] = betterproto.message_field(6)
    is_allowed_add_to_other_villa: bool = betterproto.bool_field(7)


@dataclass
class RobotTemplateParam(betterproto.Message):
    desc: str = betterproto.string_field(1)


@dataclass
class RobotTemplateCommand(betterproto.Message):
    name: str = betterproto.string_field(1)
    desc: str = betterproto.string_field(2)
    params: List["RobotTemplateParam"] = betterproto.message_field(3)


@dataclass
class RobotTemplateCustomSetting(betterproto.Message):
    name: str = betterproto.string_field(1)
    url: str = betterproto.string_field(2)


@dataclass
class Robot(betterproto.Message):
    template: "RobotTemplate" = betterproto.message_field(1)
    villa_id: int = betterproto.uint64_field(2)


@dataclass
class QuoteMessageInfo(betterproto.Message):
    content: str = betterproto.string_field(1)
    msg_uid: str = betterproto.string_field(2)
    send_at: int = betterproto.int64_field(3)
    msg_type: str = betterproto.string_field(4)
    bot_msg_id: str = betterproto.string_field(5)
    from_user_id: int = betterproto.uint64_field(6)
    from_user_id_str: str = betterproto.string_field(7)
    from_user_nickname: str = betterproto.string_field(8)


class RobotEventEventType(betterproto.Enum):
    UnknowRobotEventType = 0
    JoinVilla = 1
    SendMessage = 2
    CreateRobot = 3
    DeleteRobot = 4
    AddQuickEmoticon = 5
    AuditCallback = 6
    ClickMsgComponent = 7


@dataclass
class RobotEvent(betterproto.Message):
    robot: "Robot" = betterproto.message_field(1)
    type: "RobotEventEventType" = betterproto.enum_field(2)
    extend_data: "RobotEventExtendData" = betterproto.message_field(3)
    created_at: int = betterproto.int64_field(4)
    id: str = betterproto.string_field(5)
    send_at: int = betterproto.int64_field(6)


@dataclass
class RobotEventExtendData(betterproto.Message):
    join_villa: "RobotEventExtendDataJoinVillaInfo" = betterproto.message_field(
        1,
        group="event_data",
    )
    send_message: "RobotEventExtendDataSendMessageInfo" = betterproto.message_field(
        2,
        group="event_data",
    )
    create_robot: "RobotEventExtendDataCreateRobotInfo" = betterproto.message_field(
        3,
        group="event_data",
    )
    delete_robot: "RobotEventExtendDataDeleteRobotInfo" = betterproto.message_field(
        4,
        group="event_data",
    )
    add_quick_emoticon: (
        "RobotEventExtendDataAddQuickEmoticonInfo"
    ) = betterproto.message_field(5, group="event_data")
    audit_callback: "RobotEventExtendDataAuditCallbackInfo" = betterproto.message_field(
        6,
        group="event_data",
    )
    click_msg_component: (
        "RobotEventExtendDataClickMsgComponentInfo"
    ) = betterproto.message_field(7, group="event_data")


@dataclass
class RobotEventExtendDataJoinVillaInfo(betterproto.Message):
    join_uid: int = betterproto.uint64_field(1)
    join_user_nickname: str = betterproto.string_field(2)
    join_at: int = betterproto.int64_field(3)
    villa_id: int = betterproto.uint64_field(4)


class ObjectName(betterproto.Enum):
    UnknowObjectName = 0
    Text = 1
    Post = 2


@dataclass
class RobotEventExtendDataSendMessageInfo(betterproto.Message):
    content: str = betterproto.string_field(1)
    from_user_id: int = betterproto.uint64_field(2)
    send_at: int = betterproto.int64_field(3)
    object_name: ObjectName = betterproto.enum_field(4)
    room_id: int = betterproto.uint64_field(5)
    nickname: str = betterproto.string_field(6)
    msg_uid: str = betterproto.string_field(7)
    bot_msg_id: str = betterproto.string_field(8)
    villa_id: int = betterproto.uint64_field(9)
    quote_msg: "QuoteMessageInfo" = betterproto.message_field(10)


@dataclass
class RobotEventExtendDataCreateRobotInfo(betterproto.Message):
    villa_id: int = betterproto.uint64_field(1)


@dataclass
class RobotEventExtendDataDeleteRobotInfo(betterproto.Message):
    villa_id: int = betterproto.uint64_field(1)


@dataclass
class RobotEventExtendDataAddQuickEmoticonInfo(betterproto.Message):
    villa_id: int = betterproto.uint64_field(1)
    room_id: int = betterproto.uint64_field(2)
    uid: int = betterproto.uint64_field(3)
    emoticon_id: int = betterproto.uint32_field(4)
    emoticon: str = betterproto.string_field(5)
    msg_uid: str = betterproto.string_field(6)
    is_cancel: bool = betterproto.bool_field(7)
    bot_msg_id: str = betterproto.string_field(8)
    emoticon_type: int = betterproto.uint32_field(9)


class RobotEventExtendDataAuditCallbackInfoAuditResult(betterproto.Enum):
    None_ = 0
    Pass = 1
    Reject = 2


@dataclass
class RobotEventExtendDataAuditCallbackInfo(betterproto.Message):
    audit_id: str = betterproto.string_field(1)
    bot_tpl_id: str = betterproto.string_field(2)
    villa_id: int = betterproto.uint64_field(3)
    room_id: int = betterproto.uint64_field(4)
    user_id: int = betterproto.uint64_field(5)
    pass_through: str = betterproto.string_field(6)
    audit_result: (
        RobotEventExtendDataAuditCallbackInfoAuditResult
    ) = betterproto.enum_field(7)


@dataclass
class RobotEventExtendDataClickMsgComponentInfo(betterproto.Message):
    villa_id: int = betterproto.uint64_field(1)
    room_id: int = betterproto.uint64_field(2)
    component_id: str = betterproto.string_field(3)
    msg_uid: str = betterproto.string_field(4)
    uid: int = betterproto.uint64_field(5)
    bot_msg_id: str = betterproto.string_field(6)
    template_id: int = betterproto.uint64_field(7)
    extra: str = betterproto.string_field(8)


@dataclass
class RobotEventMessage(betterproto.Message):
    event: "RobotEvent" = betterproto.message_field(1)



from typing import List, Literal, Optional

from pydantic import BaseModel, Extra, Field


class BotInfo(BaseModel):
    bot_id: str
    bot_secret: str
    connection_type: Literal["webhook", "websocket"] = "webhook"
    test_villa_id: Optional[int] = None
    pub_key: str
    callback_url: Optional[str] = None
    verify_event: bool = True


class Config(BaseModel, extra=Extra.ignore):
    villa_bots: List[BotInfo] = Field(default_factory=list)



from dataclasses import asdict
from datetime import datetime
from enum import IntEnum
import json
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import Annotated, override

from nonebot.adapters import Event as BaseEvent
from nonebot.utils import escape_tag

import betterproto
from pydantic import Field, root_validator

from .message import Message, MessageSegment
from .models import MessageContentInfoGet, QuoteMessage, Robot
from .payload import RobotEvent


class EventType(IntEnum):
    """事件类型"""

    JoinVilla = 1
    SendMessage = 2
    CreateRobot = 3
    DeleteRobot = 4
    AddQuickEmoticon = 5
    AuditCallback = 6
    ClickMsgComponent = 7


class AuditResult(IntEnum):
    """审核结果类型"""

    Compatibility = 0
    """兼容"""
    Pass = 1
    """通过"""
    Reject = 2
    """驳回"""


class Event(BaseEvent):
    """Villa 事件基类"""

    robot: Robot
    """用户机器人访问凭证"""
    type: EventType
    """事件类型"""
    id: str
    """事件 id"""
    created_at: int
    """事件创建时间"""
    send_at: int
    """事件回调时间"""

    @property
    def bot_id(self) -> str:
        """机器人ID"""
        return self.robot.template.id

    @property
    def time(self) -> datetime:
        """事件创建的时间"""
        return datetime.fromtimestamp(self.created_at)

    @override
    def get_event_name(self) -> str:
        return self.type.name

    @override
    def get_event_description(self) -> str:
        return escape_tag(repr(self.dict()))

    @override
    def get_message(self):
        raise ValueError("Event has no message!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def get_session_id(self) -> str:
        return f"{self.robot.villa_id}_{self.id}"

    @override
    def is_tome(self) -> bool:
        return False


class NoticeEvent(Event):
    """通知事件"""

    @override
    def get_type(self) -> str:
        return "notice"


class MessageEvent(Event):
    """消息事件

    但目前大别野只有SendMessageEvent这一种消息事件，所以推荐直接使用SendMessageEvent"""

    @override
    def get_type(self) -> str:
        return "message"


class JoinVillaEvent(NoticeEvent):
    """新用户加入大别野事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###JoinVilla"""

    type: Literal[EventType.JoinVilla] = EventType.JoinVilla
    join_uid: int
    """用户ID"""
    join_user_nickname: str
    """用户昵称"""
    join_at: int
    """用户加入时间的时间戳"""
    villa_id: int
    """大别野 ID"""

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            (
                f"User(nickname={self.join_user_nickname},id={self.join_uid}) "
                f"join Villa(id={self.villa_id})"
            ),
        )

    @override
    def get_user_id(self) -> str:
        return str(self.join_uid)

    @override
    def get_session_id(self) -> str:
        return f"{self.villa_id}_{self.join_uid}"


class SendMessageEvent(MessageEvent):
    """用户@机器人发送消息事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###SendMessage"""

    type: Literal[EventType.SendMessage] = EventType.SendMessage
    content: MessageContentInfoGet
    """消息内容"""
    from_user_id: int
    """发送者ID"""
    send_at: int
    """发送时间的时间戳"""
    room_id: int
    """房间ID"""
    object_name: int
    """目前只支持文本类型消息"""
    nickname: str
    """用户昵称"""
    msg_uid: str
    """消息ID"""
    bot_msg_id: Optional[str] = None
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""
    villa_id: int
    """大别野 ID"""
    quote_msg: Optional[QuoteMessage] = None
    """回调消息引用消息的基础信息"""

    to_me: bool = True
    """是否和Bot有关"""
    message: Message
    """事件消息"""
    original_message: Message
    """事件原始消息"""

    @property
    def message_id(self) -> str:
        """消息ID"""
        return self.msg_uid

    @property
    def reply(self) -> Optional[QuoteMessage]:
        """消息的回复信息"""
        return self.quote_msg

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            (
                f"Message(id={self.msg_uid}) was sent from"
                f" User(nickname={self.nickname}, id={self.from_user_id}) in"
                f" Room(id={self.room_id}) of Villa(id={self.villa_id}),"
                f" content={repr(self.message)}"
            ),
        )

    @override
    def get_message(self) -> Message:
        """获取事件消息"""
        return self.message

    @override
    def is_tome(self) -> bool:
        """是否和Bot有关"""
        return self.to_me

    @override
    def get_user_id(self) -> str:
        """获取用户ID"""
        return str(self.from_user_id)

    @override
    def get_session_id(self) -> str:
        """获取会话ID"""
        return f"{self.villa_id}_{self.room_id}_{self.from_user_id}"

    @root_validator(pre=True)
    @classmethod
    def payload_to_event(cls, data: Dict[str, Any]):
        if not data.get("content"):
            return data
        msg = Message()
        msg_content_info = data["content"] = json.loads(data["content"])
        if quote := msg_content_info.get("quote"):
            msg.append(
                MessageSegment.quote(
                    message_id=quote["quoted_message_id"],
                    message_send_time=quote["quoted_message_send_time"],
                ),
            )

        content = msg_content_info["content"]
        text = content["text"]
        entities = content["entities"]
        if not entities:
            msg.append(MessageSegment.text(text))
            data["message"] = data["original_message"] = msg
            return data
        text = text.encode("utf-16")
        last_offset: int = 0
        last_length: int = 0
        for entity in entities:
            end_offset: int = last_offset + last_length
            offset: int = entity["offset"]
            length: int = entity["length"]
            entity_detail = entity["entity"]
            if offset != end_offset:
                msg.append(
                    MessageSegment.text(
                        text[((end_offset + 1) * 2) : ((offset + 1) * 2)].decode(
                            "utf-16",
                        ),
                    ),
                )
            entity_text = text[(offset + 1) * 2 : (offset + length + 1) * 2].decode(
                "utf-16",
            )
            if entity_detail["type"] == "mentioned_robot":
                entity_detail["bot_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_robot(
                        entity_detail["bot_id"],
                        entity_detail["bot_name"],
                    ),
                )
            elif entity_detail["type"] == "mentioned_user":
                entity_detail["user_name"] = entity_text.lstrip("@")[:-1]
                msg.append(
                    MessageSegment.mention_user(
                        int(entity_detail["user_id"]),
                        entity_detail["user_name"],
                        villa_id=data["villa_id"],
                    ),
                )
            elif entity_detail["type"] == "mention_all":
                entity_detail["show_text"] = entity_text.lstrip("@")[:-1]
                msg.append(MessageSegment.mention_all(entity_detail["show_text"]))
            elif entity_detail["type"] == "villa_room_link":
                entity_detail["room_name"] = entity_text.lstrip("#")[:-1]
                msg.append(
                    MessageSegment.room_link(
                        int(entity_detail["villa_id"]),
                        int(entity_detail["room_id"]),
                    ),
                )
            elif entity_detail["type"] == "style":
                msg.append(
                    MessageSegment.text(
                        entity_text,
                        **{entity_detail["font_style"]: True},
                    ),
                )
            else:
                entity_detail["show_text"] = entity_text
                msg.append(
                    MessageSegment.link(
                        entity_detail["url"],
                        entity_detail["show_text"],
                    ),
                )
            last_offset = offset
            last_length = length
        end_offset = last_offset + last_length
        if last_text := text[(end_offset + 1) * 2 :].decode("utf-16"):
            msg.append(MessageSegment.text(last_text))
        data["message"] = data["original_message"] = msg
        return data


class CreateRobotEvent(NoticeEvent):
    """大别野添加机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###CreateRobot"""

    type: Literal[EventType.CreateRobot] = EventType.CreateRobot
    villa_id: int
    """大别野ID"""

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Bot(id={self.bot_id}) was added to Villa(id={self.villa_id})",
        )

    @override
    def is_tome(self) -> bool:
        return True

    @override
    def get_session_id(self) -> str:
        return f"{self.villa_id}_{self.bot_id}"


class DeleteRobotEvent(NoticeEvent):
    """大别野删除机器人实例事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html###DeleteRobot"""

    type: Literal[EventType.DeleteRobot] = EventType.DeleteRobot
    villa_id: int
    """大别野ID"""

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Bot(id={self.bot_id}) was removed from Villa(id={self.villa_id})",
        )

    @override
    def is_tome(self) -> bool:
        return True

    @override
    def get_session_id(self) -> str:
        return f"{self.villa_id}_{self.bot_id}"


class AddQuickEmoticonEvent(NoticeEvent):
    """用户使用表情回复消息表态事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AddQuickEmoticon"""

    type: Literal[EventType.AddQuickEmoticon] = EventType.AddQuickEmoticon
    villa_id: int
    """大别野ID"""
    room_id: int
    """房间ID"""
    uid: int
    """发送表情的用户ID"""
    emoticon_id: int
    """表情ID"""
    emoticon: str
    """表情内容"""
    msg_uid: str
    """被回复的消息 id"""
    bot_msg_id: Optional[str] = None
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""
    is_cancel: bool = False
    """是否是取消表情"""
    emoticon_type: int
    """表情类型"""

    @override
    def get_user_id(self) -> str:
        return str(self.uid)

    @override
    def is_tome(self) -> bool:
        return True

    @override
    def get_session_id(self) -> str:
        return (
            f"{self.villa_id}_{self.room_id}_{self.uid}"
            f"_{self.emoticon_id}_{self.is_cancel}"
        )

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            (
                f"Emoticon(name={self.emoticon}, id={self.emoticon_id}) was "
                f"{'removed from' if self.is_cancel else 'added to'} "
                f"Message(id={self.msg_uid}) by User(id={self.uid}) in "
                f"Room(id=Villa(id={self.room_id}) of Villa(id={self.villa_id})"
            ),
        )


class AuditCallbackEvent(NoticeEvent):
    """审核结果回调事件

    see https://webstatic.mihoyo.com/vila/bot/doc/callback.html#AuditCallback"""

    type: Literal[EventType.AuditCallback] = EventType.AuditCallback
    audit_id: str
    """审核事件 id"""
    bot_tpl_id: str
    """机器人 id"""
    villa_id: int
    """大别野 ID"""
    room_id: int
    """房间 id（和审核接口调用方传入的值一致）"""
    user_id: int
    """用户 id（和审核接口调用方传入的值一致）"""
    pass_through: str
    """透传数据（和审核接口调用方传入的值一致）"""
    audit_result: AuditResult
    """审核结果"""

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    @override
    def is_tome(self) -> bool:
        return self.bot_id == self.bot_tpl_id

    @override
    def get_session_id(self) -> str:
        return f"{self.villa_id}_{self.bot_tpl_id}_{self.audit_id}"

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            (
                f"Audit(id={self.audit_id},result={self.audit_result}) of "
                f"User(id={self.user_id}) in Room(id={self.room_id}) of "
                f"Villa(id={self.villa_id})"
            ),
        )


class ClickMsgComponentEvent(NoticeEvent):
    type: Literal[EventType.ClickMsgComponent] = EventType.ClickMsgComponent
    villa_id: int
    """大别野 ID"""
    room_id: int
    """房间 ID"""
    uid: int
    """用户 ID"""
    msg_uid: str
    """消息 ID"""
    bot_msg_id: Optional[str] = None
    """如果被回复的消息从属于机器人，则该字段不为空字符串"""
    component_id: str
    """机器人自定义的组件id"""
    template_id: int
    """如果该组件模板为已创建模板，则template_id不为0"""
    extra: str
    """机器人自定义透传信息"""


event_classes = Annotated[
    Union[
        JoinVillaEvent,
        SendMessageEvent,
        CreateRobotEvent,
        DeleteRobotEvent,
        AddQuickEmoticonEvent,
        AuditCallbackEvent,
    ],
    Field(discriminator="type"),
]


def pre_handle_webhook_event(payload: Dict[str, Any]):
    if (event_type := EventType._value2member_map_.get(payload["type"])) is None:
        raise ValueError(
            f"Unknown event type: {payload['type']} data={escape_tag(str(payload))}",
        )
    event_name = event_type.name
    if event_name not in payload["extend_data"]["EventData"]:
        raise ValueError("Cannot find event data for event type: {event_name}")
    payload.update(payload["extend_data"]["EventData"][event_name])
    payload.pop("extend_data")
    return payload


def pre_handle_event_websocket(payload: RobotEvent):
    event_data = asdict(payload)
    event_data.update(
        event_data["extend_data"][
            betterproto.Casing.SNAKE(EventType(event_data["type"]).name)
        ],
    )
    event_data.pop("extend_data")
    return event_data


__all__ = [
    "Event",
    "NoticeEvent",
    "MessageEvent",
    "JoinVillaEvent",
    "SendMessageEvent",
    "CreateRobotEvent",
    "DeleteRobotEvent",
    "AddQuickEmoticonEvent",
    "AuditCallbackEvent",
]



from dataclasses import dataclass
from typing import TYPE_CHECKING, Iterable, List, Literal, Optional, Type, Union
from typing_extensions import Self, TypedDict, override

from nonebot.adapters import (
    Message as BaseMessage,
    MessageSegment as BaseMessageSegment,
)

from .models import (
    Badge,
    Component,
    Image,
    ImageMessageContent,
    Link,
    MentionedAll,
    MentionedRobot,
    MentionedUser,
    MessageContentInfo,
    Panel,
    PostMessageContent,
    PreviewLink,
    QuoteInfo,
    TextMessageContent,
    VillaRoomLink,
)


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @override
    def __repr__(self) -> str:
        return self.__str__()

    @override
    def __add__(
        self,
        other: Union[str, "MessageSegment", Iterable["MessageSegment"]],
    ) -> "Message":
        return super().__add__(other)

    @override
    def __radd__(
        self,
        other: Union[str, "MessageSegment", Iterable["MessageSegment"]],
    ) -> "Message":
        return super().__radd__(other)

    @override
    def is_text(self) -> bool:
        return self.type == "text"

    @staticmethod
    def text(
        text: str,
        bold: bool = False,
        italic: bool = False,
        strikethrough: bool = False,
        underline: bool = False,
    ) -> "TextSegment":
        """纯文本消息段

        参数:
            text: 文本内容

        返回:
            TextSegment: 消息段对象
        """
        return TextSegment(
            "text",
            {
                "text": text,
                "bold": bold,
                "italic": italic,
                "strikethrough": strikethrough,
                "underline": underline,
            },
        )

    @staticmethod
    def mention_robot(bot_id: str, bot_name: str) -> "MentionRobotSegement":
        """@机器人消息段

        参数:
            bot_id: 机器人ID
            bot_name: 机器人的名字

        返回:
            MentionRobotSegement: 消息段对象
        """
        return MentionRobotSegement(
            "mention_robot",
            {"mention_robot": MentionedRobot(bot_id=bot_id, bot_name=bot_name)},
        )

    @staticmethod
    def mention_user(
        user_id: int,
        user_name: Optional[str] = None,
        villa_id: Optional[int] = None,
    ) -> "MentionUserSegement":
        """@用户消息段

        user_name和villa_id必须有其中之一

        参数:
            user_id: 用户ID
            user_name: 用户名称
            villa_id: 用户所在大别野ID

        返回:
            MentionUserSegement: 消息段对象
        """
        if not (user_name or villa_id):
            raise ValueError("user_name and villa_id must have one of them")
        return MentionUserSegement(
            "mention_user",
            {
                "mention_user": MentionedUser(
                    user_id=str(user_id),
                    user_name=user_name,
                ),
                "villa_id": villa_id,
            },
        )

    @staticmethod
    def mention_all(show_text: str = "全体成员") -> "MentionAllSegement":
        """@全体成员消息段

        参数:
            show_text: 展示文本. 默认为 "全体成员".

        返回:
            MentionAllSegement: 消息段对象
        """
        return MentionAllSegement(
            "mention_all",
            {"mention_all": MentionedAll(show_text=show_text)},
        )

    @staticmethod
    def room_link(
        villa_id: int,
        room_id: int,
        room_name: Optional[str] = None,
    ) -> "RoomLinkSegment":
        """房间链接消息段，点击后可以跳转到指定房间

        参数:
            villa_id: 大别野ID
            room_id: 房间ID

        返回:
            VillaRoomLinkSegment: 消息段对象
        """
        return RoomLinkSegment(
            "room_link",
            {
                "room_link": VillaRoomLink(
                    villa_id=str(villa_id),
                    room_id=str(room_id),
                    room_name=room_name,
                ),
            },
        )

    @staticmethod
    def link(
        url: str,
        show_text: Optional[str] = None,
        requires_bot_access_token: bool = False,
    ) -> "LinkSegment":
        """链接消息段，使用该消息段才能让链接可以直接点击进行跳转

        参数:
            url: 链接
            show_text: 链接显示的文本

        返回:
            LinkSegment: 消息段对象
        """
        return LinkSegment(
            "link",
            {
                "link": Link(
                    url=url,
                    show_text=show_text or url,
                    requires_bot_access_token=requires_bot_access_token,
                ),
            },
        )

    @staticmethod
    def quote(message_id: str, message_send_time: int) -> "QuoteSegment":
        """引用(回复)消息段

        参数:
            message_id: 被引用的消息ID
            message_send_time: 被引用的消息发送时间

        返回:
            QuoteSegment: 消息段对象
        """
        return QuoteSegment(
            "quote",
            {
                "quote": QuoteInfo(
                    quoted_message_id=message_id,
                    quoted_message_send_time=message_send_time,
                    original_message_id=message_id,
                    original_message_send_time=message_send_time,
                ),
            },
        )

    @staticmethod
    def image(
        url: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
        file_size: Optional[int] = None,
    ) -> "ImageSegment":
        """图片消息段

        参数:
            url: 图片链接
            width: 图片宽度
            height: 图片高度
            file_size: 图片大小

        返回:
            ImageSegment: 消息段对象
        """
        return ImageSegment(
            "image",
            {
                "image": Image.parse_obj(
                    {
                        "url": url,
                        "size": {
                            "width": width,
                            "height": height,
                        },
                        "file_size": file_size,
                    },
                ),
            },
        )

    @staticmethod
    def post(post_id: str) -> "PostSegment":
        """帖子转发消息段

        参数:
            post_id: 帖子ID

        返回:
            PostSegment: 消息段对象
        """
        return PostSegment("post", {"post": PostMessageContent(post_id=post_id)})

    @staticmethod
    def preview_link(
        icon_url: str,
        image_url: str,
        is_internal_link: bool,
        title: str,
        content: str,
        url: str,
        source_name: str,
    ) -> "PreviewLinkSegment":
        """预览链接(卡片)消息段

        参数:
            icon_url: 图标链接
            image_url: 封面链接
            is_internal_link: 是否为官方
            title: 标题
            content: 内容
            url: 链接
            source_name: 来源

        返回:
            PreviewLinkSegment: 消息段对象
        """
        return PreviewLinkSegment(
            "preview_link",
            {
                "preview_link": PreviewLink(
                    icon_url=icon_url,
                    image_url=image_url,
                    is_internal_link=is_internal_link,
                    title=title,
                    content=content,
                    url=url,
                    source_name=source_name,
                ),
            },
        )

    @staticmethod
    def badge(
        icon_url: str,
        text: str,
        url: str,
    ) -> "BadgeSegment":
        """消息下方徽标

        参数:
            icon_url: 图标链接
            text: 文本
            url: 链接

        返回:
            BadgeSegment: 消息段对象
        """
        return BadgeSegment(
            "badge",
            {
                "badge": Badge(
                    icon_url=icon_url,
                    text=text,
                    url=url,
                ),
            },
        )

    @staticmethod
    def components(*components: Component) -> "ComponentsSegment":
        return ComponentsSegment(
            "components",
            {
                "components": list(components),
            },
        )

    @staticmethod
    def panel(panel: Union[Panel, int]) -> "PanelSegment":
        if isinstance(panel, int):
            panel = Panel(template_id=panel)
        return PanelSegment(
            "panel",
            {
                "panel": panel,
            },
        )


class TextData(TypedDict):
    text: str
    bold: bool
    italic: bool
    strikethrough: bool
    underline: bool


@dataclass
class TextSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["text"]
        data: TextData

    @override
    def __str__(self) -> str:
        return self.data["text"]


class MentionRobotData(TypedDict):
    mention_robot: MentionedRobot


@dataclass
class MentionRobotSegement(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["mention_robot"]
        data: MentionRobotData

    @override
    def __str__(self) -> str:
        return f"<mention_robot:{self.data['mention_robot'].bot_id}>"


class MentionUserData(TypedDict):
    mention_user: MentionedUser
    villa_id: Optional[int]


@dataclass
class MentionUserSegement(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["mention_user"]
        data: MentionUserData

    @override
    def __str__(self) -> str:
        return f"<mention_user:{self.data['mention_user'].user_id}>"


class MentionAllData(TypedDict):
    mention_all: MentionedAll


@dataclass
class MentionAllSegement(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["mention_all"]
        data: MentionAllData

    @override
    def __str__(self) -> str:
        return f"<mention_all:{self.data['mention_all'].show_text}>"


class RoomLinkData(TypedDict):
    room_link: VillaRoomLink


@dataclass
class RoomLinkSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["room_link"]
        data: RoomLinkData

    @override
    def __str__(self) -> str:
        return f"<room_link:{self.data['room_link'].room_id}>"


class LinkData(TypedDict):
    link: Link


@dataclass
class LinkSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["link"]
        data: LinkData

    @override
    def __str__(self) -> str:
        return f"<link:{self.data['link'].url}>"


class ImageData(TypedDict):
    image: Image


@dataclass
class ImageSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["image"]
        data: ImageData

    @override
    def __str__(self) -> str:
        return f"<image:{self.data['image'].url}>"


class QuoteData(TypedDict):
    quote: QuoteInfo


@dataclass
class QuoteSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["quote"]
        data: QuoteData

    @override
    def __str__(self) -> str:
        return f"<quote:{self.data['quote'].quoted_message_id}>"


class PostData(TypedDict):
    post: PostMessageContent


@dataclass
class PostSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["post"]
        data: PostData

    @override
    def __str__(self) -> str:
        return f"<post:{self.data['post'].post_id}>"


class PreviewLinkData(TypedDict):
    preview_link: PreviewLink


@dataclass
class PreviewLinkSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["preview_link"]
        data: PreviewLinkData

    @override
    def __str__(self) -> str:
        return f"<preview_link:{self.data['preview_link'].title}>"


class BadgeData(TypedDict):
    badge: Badge


@dataclass
class BadgeSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["badge"]
        data: BadgeData

    @override
    def __str__(self) -> str:
        return f"<badge:{self.data['badge'].text}>"


class ComponentsData(TypedDict):
    components: List[Component]


@dataclass
class ComponentsSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["components"]
        data: ComponentsData

    @override
    def __str__(self) -> str:
        return f"<components:{self.data['components']}>"


class PanelData(TypedDict):
    panel: Panel


@dataclass
class PanelSegment(MessageSegment):
    if TYPE_CHECKING:
        type: Literal["panel"]
        data: PanelData

    @override
    def __str__(self) -> str:
        return f"<panel:{self.data['panel']}>"


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @override
    def __add__(
        self,
        other: Union[str, MessageSegment, Iterable[MessageSegment]],
    ) -> "Message":
        return super().__add__(
            MessageSegment.text(other) if isinstance(other, str) else other,
        )

    @override
    def __radd__(
        self,
        other: Union[str, MessageSegment, Iterable[MessageSegment]],
    ) -> "Message":
        return super().__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other,
        )

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        yield MessageSegment.text(msg)

    @classmethod
    def from_message_content_info(cls, content_info: MessageContentInfo) -> Self:
        msg = cls()
        if content_info.quote:
            msg.append(
                MessageSegment.quote(
                    content_info.quote.quoted_message_id,
                    content_info.quote.quoted_message_send_time,
                ),
            )
        content = content_info.content
        if isinstance(content, TextMessageContent):
            if not content.entities:
                msg.append(MessageSegment.text(content.text))
                return msg
            text = content.text.encode("utf-16")
            last_offset: int = 0
            last_length: int = 0
            for entity in content.entities:
                end_offset = last_offset + last_length
                offset = entity.offset
                length = entity.length
                entity_detail = entity.entity
                if offset != end_offset:
                    msg.append(
                        MessageSegment.text(
                            text[((end_offset + 1) * 2) : ((offset + 1) * 2)].decode(
                                "utf-16",
                            ),
                        ),
                    )
                if entity_detail.type == "mentioned_robot":
                    msg.append(
                        MessageSegment.mention_robot(
                            entity_detail.bot_id,
                            entity_detail.bot_name,
                        ),
                    )
                elif entity_detail.type == "mentioned_user":
                    msg.append(
                        MessageSegment.mention_user(
                            int(entity_detail.user_id),
                            entity_detail.user_name,
                        ),
                    )
                elif entity_detail.type == "mention_all":
                    msg.append(MessageSegment.mention_all(entity_detail.show_text))
                elif entity_detail.type == "villa_room_link":
                    msg.append(
                        MessageSegment.room_link(
                            int(entity_detail.villa_id),
                            int(entity_detail.room_id),
                            entity_detail.room_name,
                        ),
                    )
                elif entity_detail.type == "style":
                    msg.append(
                        MessageSegment.text(
                            text[
                                ((offset + 1) * 2) : ((offset + length + 1) * 2)
                            ].decode(
                                "utf-16",
                            ),
                            **{entity_detail.font_style: True},
                        ),
                    )
                else:
                    msg.append(
                        MessageSegment.link(entity_detail.url, entity_detail.show_text),
                    )
                last_offset = offset
                last_length = length
            end_offset = last_offset + last_length
            if last_text := text[(end_offset + 1) * 2 :].decode("utf-16"):
                msg.append(MessageSegment.text(last_text))
            return msg
        elif isinstance(content, ImageMessageContent):
            msg.append(
                MessageSegment.image(
                    content.url,
                    content.size.width if content.size else None,
                    content.size.height if content.size else None,
                    content.file_size,
                ),
            )
            return msg
        else:
            msg.append(MessageSegment.post(content.post_id))
            return msg
